/***************************************************************************
                          aviaudio.cpp  -  description
                             -------------------
    begin                : Fri Nov 23 2001
    copyright            : (C) 2001 by mean
    email                : fixounet@free.fr

This class deals with a chunked / not chunked stream
It is an fopen/fwrite lookalike interface to chunks



 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <stream.h>
#include <assert.h>
#include <math.h>

#include "avifmt.h"
#include "avifmt2.h"
#include "avio.hxx"
#include "fourcc.h"
#include "riffparser.h"
#include "subchunk.h"
#include "indexer.hxx"
#include "aviaudio.hxx"
#include "ADM_audiocodec/ADM_audiocodec.h"


//___________________________________
//
//___________________________________
AVDMAviAudioStream::AVDMAviAudioStream(ShortIndex *idx,
						uint32_t nbchunk,
						riffparser * riff,
				       		WAVHeader * wav, 
						uint32_t preload,
						uint32_t extraLen,
						uint8_t *extraData
						
						)      : AVDMGenericAudioStream()
{
    UNUSED_ARG(preload);
//     uint32_t j;
    _extraLen=extraLen;
    _extraData=extraData;
    if(_extraLen)
    	{
		printf(" we have %lu bytes of extra data in wavheader\n",_extraLen);
	}
    _pos=0;
    _index=idx;
    _destroyable = 0;
    _nb_chunks=nbchunk;
    strcpy(_name, "FILE:AVI");

     _codec=getAudioCodec(wav->encoding,wav,_extraLen,_extraData);
     assert(_codec);

    _current_index = 0;

    printf("\n Audio streamer initialized");
    _abs_position = 0;
    _rel_position = 0;
    _riff = riff;
    _wavheader = wav;
    _length = 0;

    for(uint32_t i=0;i<_nb_chunks;i++)
    	{
        	_length+=_index[i].Length;
       }
	
    printf("\n Total audio length : %lu",_length);
}
//#define VERBOSE_L3

//___________________________________
//
//___________________________________
uint32_t AVDMAviAudioStream::read(uint32_t len, uint8_t * buffer)
{
    uint32_t togo;
    uint32_t avail, rd;
//    uint32_t askedlen=len;

    // just to be sure....  
    _riff->goTo(_abs_position);	// go to the beginning of the current schunk
    _riff->forwardbyte(_rel_position);	// go to the beginning of the current schunk

    togo = len;

#ifdef VERBOSE_L3
    printf("\n ABS: %lu rel:%lu len:%lu", _abs_position, _rel_position,
	   len);
#endif

     do
     {
	  avail = _index[_current_index].Length - _rel_position;	// how much available ?

	  if (avail > togo)	// we can grab all in one move
	    {

		if(!(togo == _riff->read(togo, (uint8_t *) buffer)))
		{
			printf("\n Badly indexed file, trying to continue anyway...\n");
			 return 1;
		}
#ifdef VERBOSE_L3

		printf("\n REL: %lu rel:%lu len:%lu", _abs_position,
		       _rel_position, togo);
#endif

		_rel_position += togo;
#ifdef VERBOSE_L3

		printf("\n FINISH: %lu rel:%lu len:%lu", _abs_position,
		       _rel_position, togo);
#endif

		buffer += togo;
		togo = 0;
	  } else		// read the whole subchunk and go to next one
	    {
#ifdef VERBOSE_L3

		printf("\n CONT: %lu rel:%lu len:%lu", _abs_position,
		       _rel_position, togo);
#endif

		rd = _riff->read(avail, buffer);
		if (rd != avail)
		  {
		      printf("\n Error : Expected :%lu bytes read :%lu \n",     rd, avail);
		      assert(0);

		  }
		buffer += avail;
		togo -= avail;

		_current_index++;
		if (_current_index>=_nb_chunks)
		  {
#ifdef VERBOSE_L3

		printf("\n OVR: %lu rel:%lu len:%lu", _abs_position,
		       _rel_position, togo);
#endif
		    _abs_position =_index[0].Offset ;
            	     _rel_position = 0;
		     _current_index=0;
		      assert(len >= togo);
        	      _pos+=len;
                      _pos-=togo;
		      return (len - togo);

		  }
    	else
     	{
#ifdef VERBOSE_L3
		printf("\n CONT: %lu rel:%lu len:%lu", _abs_position,
		       _rel_position, togo);
#endif
		_abs_position = _index[_current_index].Offset;
		_rel_position = 0;
		_riff->goTo(_abs_position);
	    }
      }
    }
    while (togo);
    _pos+=len;
    return len;

}

//_______________________________________________
//
//_______________________________________________
uint8_t AVDMAviAudioStream::goTo(uint32_t newoffset)
{
    uint32_t len;
    len = newoffset;
    if(len>=_length)
    	{
       	printf("\n AudioGoto Out of bound !\n");
     		return 0;
       }
    _current_index = 0;	// start at beginning
#ifdef VERBOSE_L3
    printf("\n Stream offset : %lu\n", newoffset);
#endif
    do
      {
	  if (len >= _index[_current_index].Length)	// skip whole subchunk
	    {
		len -= _index[_current_index].Length;
		_current_index++;
  		if(_current_index>=_nb_chunks)
    			{
                  printf("\n idx : %lu max: %lu len:%lu\n",  _current_index,_nb_chunks,len);
		  _current_index=0;
                  //assert(0);
                  //pos=0;
                  return 0;
       		};
		_rel_position = 0;
	  } else		// we got the last one
	    {
		_riff->goTo(_index[_current_index].Offset);
		_rel_position = len;

		_riff->forwardbyte(len);

		len = 0;

	    }
	  //printf("\n %lu len bytes to go",len);
      }
    while (len);
    _abs_position = _index[_current_index].Offset;
  assert(_current_index<_nb_chunks);
    _pos=newoffset;
    return 1;
}



//___________________
