/***************************************************************************
                          aviio.cpp  -  description
                             -------------------
    begin                : Thu Nov 1 2001
    copyright            : (C) 2001 by mean
    email                : fixounet@free.fr

		This part of code deal with handling a avi file as a whole

		You can
			- open it
			- save the sound track
			- get frames (next, previous,any,next key frame, previous ket frame)
			- save AVI

		It is heavily based on the class riffparser which handles the low
		level file io.

		The best is to well understand riffparser before going into more details

		ATM, the file is more oriented toward simplicity and *NOT* performance
		I don't have yet a clear idea on how the job is splitted between
		gui/aviheader/riffparser

		A small explanation on how a AVI file is build (AFAIK)

		The sub chunk entity is build as follow
			fourCC : Id (ex: "avih", "strf"....)
			uint32 : size
			....

		A chunk is build on a list

			fourcc   : "LIST"
			uint32_t : size
				subchunk 1
				subchunk 2

				junk			(junk is a particular subchunk used for padding)

		An avi file is build like this

		RIFF
		SIZE
		AVIH

		LIST hdrl
				subchunk avih  AVIMAINHEADER struct
				LIST strl for stream 1
								subchunk strh	header
								subchunk strf format
								subchunk junk
				LIST strl for stream 2
								subchunk strh	header
								subchunk strf format
								subchunk junk
				....
    		subchunk Junk

		LIST movie
				subchunk 01wb		// stream / type wb for sound;cb for video
				subchunk 00cb		// stream / type wb for sound;cb for video
				subchunk 01wb		// stream / type wb for sound;cb for video
				subchunk 00cb		// stream / type wb for sound;cb for video
				subchunk 01wb		// stream / type wb for sound;cb for video
				subchunk JUNK
		LIST idx0
				contains index for movie
				index is
					fcc 			u32		identify stream
          flags 		u32   keyframe or not
          abs 			u32   absolute position / beginning of list
          len 			u32   length of this subchunk


	The riffparser builds a table containint the address of each list/subchunk
	You can then access them by asking to riffparser

	That's as far as i know and guessed by analysing the hex dump of avifile
	I doubt the riffparser to be very solid/and or smart.

Mean.	

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <string.h>
//#include <stream.h>
#include <assert.h>
#include <math.h>

#include "config.h"
#include "avifmt.h"
#include "avifmt2.h"
#include "avio.hxx"
#include "fourcc.h"
#include "riffparser.h"
#include "subchunk.h"
#include "ADM_audio/aviaudio.hxx"
#include "ADM_toolkit/toolkit.hxx"

uint8_t setStreamInfo(uint8_t * start_ptr,
		      uint8_t * stream,
		      uint8_t * info, uint32_t infolen, uint32_t maxxed);

//
//
// For debug purpose, dump all we know about
// The avi
//_________________________________________________
void aviHeader::Dump(void)
{
    WAVHeader *wheader;
    printf("\n Main avi header :\n");
#define X_DUMP(x) printf("\n "#x":\t\t:%ld",_mainaviheader.x);
    X_DUMP(dwStreams);
    X_DUMP(dwMicroSecPerFrame) X_DUMP(dwMaxBytesPerSec);
    X_DUMP(dwPaddingGranularity);
    X_DUMP(dwFlags);
    X_DUMP(dwTotalFrames);
    X_DUMP(dwInitialFrames);

    X_DUMP(dwWidth);
    X_DUMP(dwHeight);
   

    printf("\n");

    printf("\n Max Audio chunk size : %lu", _maxaudio);
    printf("\n Max Video chunk size : %lu", _maxvideo);

    if (!_isvideopresent)
      {
	  printf( "No video stream attached\n");
    } else
      {
#undef X_DUMP
#define X_DUMP(x) printf("\n "#x":\t\t:%ld",_videostream.x);



	   printf(  "\nvideo stream attached:\n" );
	   printf(  "______________________\n" );

	  printf("\n VideoHeader Len: %ld (%d)",_videostreaminfolen,sizeof(BITMAPINFOHEADER ));
	  printf("\n Extra Data  : %ld",_videoExtraLen);
	  if(_videoExtraLen)
	  {
	  mixDump((uint8_t *) _videoExtraData, _videoExtraLen);
	  }

	  printf("\n fccType     :");
	  fourCC::print(_videostream.fccType);
	  printf("\n fccHandler :");
	  fourCC::print(_videostream.fccHandler);

	  X_DUMP(dwFlags);
	  X_DUMP(dwInitialFrames);
	  X_DUMP(dwRate);
	  X_DUMP(dwStart);
	  X_DUMP(dwSampleSize);
	  X_DUMP(dwScale);
	  X_DUMP(dwLength); 
    X_DUMP(dwQuality);
	  X_DUMP(dwSampleSize);
	  
	  // BIH
	  /*uint32_t 		biSize;
    uint32_t  	biWidth;
    uint32_t  	biHeight;
    uint16_t 		biPlanes;
    uint16_t 		biBitCount;
    uint32_t 		biCompression;
    uint32_t 		biSizeImage;
    uint32_t  	biXPelsPerMeter;
    uint32_t  	biYPelsPerMeter;
    uint32_t 		biClrUsed;
    uint32_t 		biClrImportant;*/
#undef X_DUMP
#define X_DUMP(x) printf("\n "#x":\t\t:%ld",(long int)_video_bih.x);
    		X_DUMP( biSize);
       	X_DUMP( biWidth);
       	X_DUMP( biHeight);
       	X_DUMP( biBitCount);
       	X_DUMP( biCompression);
        X_DUMP( biSizeImage);
       	X_DUMP( biXPelsPerMeter);
       	X_DUMP( biYPelsPerMeter);
       	X_DUMP( biClrUsed);
    
    
      }
                  
    if (!_isaudiopresent)
      {
	  printf( "No audio stream attached\n");
    } else
      {
#undef X_DUMP
#define X_DUMP(x) printf("\n "#x":\t\t:%ld",_audiostream.x);



	   printf(  "\naudio stream attached:\n" );
	   printf(  "______________________\n" );


	  printf("\n fccType     :");
	  fourCC::print(_audiostream.fccType);
	  printf("\n fccHandler :");
	  fourCC::print(_audiostream.fccHandler);
	  printf("\n fccHandler :0x%lx", _audiostream.fccHandler);


	  X_DUMP(dwFlags);
	  X_DUMP(dwInitialFrames);
	  X_DUMP(dwRate);
	  X_DUMP(dwScale);
	  X_DUMP(dwStart);
	  X_DUMP(dwLength);
	  X_DUMP(dwSuggestedBufferSize);
	  X_DUMP(dwQuality);
	  X_DUMP(dwSampleSize);
	  
	  
    printf("\n WavHeader Len: %ld (%d)",_audiostreaminfolen,sizeof(WAVHeader));
	  mixDump((uint8_t *) _audiostreaminfo, _audiostreaminfolen);
	  
	  

	  wheader = (WAVHeader *) _audiostreaminfo;
#undef X_DUMP
#define X_DUMP(x) printf("\n "#x":\t\t:%lu",(uint32_t)wheader->x);

	  X_DUMP(encoding);
	  X_DUMP(channels);	/* 1 = mono, 2 = stereo */
	  X_DUMP(frequency);	/* One of 11025, 22050, or 44100 Hz */
	  X_DUMP(byterate);	/* Average bytes per second */
	  X_DUMP(blockalign);	/* Bytes per sample block */
	  X_DUMP(bitspersample);	/* One of 8, 12, 16, or 4 for ADPCM */
	 if(wheader->encoding==WAV_MP3 && _audiostream.dwScale==1152)
      {
		GUI_Alert("This looks like MP3 vbr.\n Use audio->build time map");

      }

      }

}

//
//  Close avi and delete attached structures
//____________________________
uint8_t aviHeader::closeAVI(void)
{
    // first delete riff parser
    delete _riff;
    if(_isaudiopresent)
    if (_audiostreaminfo)
      {
	  free(_audiostreaminfo);
	  _audiostreaminfo = NULL;
      }
    if (_avdm_stream)
	delete _avdm_stream;

    // now parse linked list and destroy each one of them

#if 0
 SubChunk *chunk, *c2;
    chunk = _videochunks._next;
    while (chunk)
      {
	  c2 = chunk;
	  chunk = chunk->_next;
	  delete c2;
      };
#endif
    if (_videoindex)
      {
	  delete [] _videoindex;
	  _videoindex = NULL;
      }
    if (_isaudiopresent)
      {

        delete [] _audioindex;
        _audioindex=0;
#if 0
	  chunk = _audiochunks._next;
	  while (chunk)
	    {
		c2 = chunk;
		chunk = chunk->_next;
		delete c2;
	    };
#endif
      }
    _isvideopresent = 0;
    _isaudiopresent = 0;



    return 1;

}

//______________________________________
//
//   Create riff parser, open avi and
//   collect stream information
//   Note that file is still opened at the end
//______________________________________
uint8_t aviHeader::openAVI(char *name, uint8_t rebuild_index)
{
    uint8_t *buffer,res;
    uint32_t len;
    uint8_t *buffer2;
    uint32_t len2;
   uint8_t swapped=0;

	_reordered=0;
    _riff = new riffparser;
    if (!_riff->open(name))
      {
	  delete _riff;
	  _riff = NULL;
	  printf("riff open failed\n");
	  return 0;

      }
#ifdef DEBUG
    _riff->dumpChunks();
#endif

// Read Main Header chunk

    assert(_riff->
	   GetNextChunk((uint8_t *) "hdrl", (uint8_t *) "avih", &buffer,
			&len));
    assert(len == sizeof(MainAVIHeader));

    memcpy((char *) &_mainaviheader, (char *) buffer,
	   sizeof(MainAVIHeader));
    free(buffer);

    last_vchunk = NULL;

   printf( "\n Main header read\n______________\n");
   Endian_AviMainHeader(&_mainaviheader );
//_________________________
// Read first stream
//_________________________
// we assume there is at least one...
   printf(  "\n Reading first stream information\n");

    assert(_riff->
	   GetNextChunk((uint8_t *) "strl", (uint8_t *) "strh", &buffer,
			&len));
    assert(_riff->
	   GetNextChunk((uint8_t *) "strl", (uint8_t *) "strf", &buffer2,
			&len2));

	Endian_AviStreamHeader((AVIStreamHeader *)buffer);
	Endian_BitMapInfo( (BITMAPINFOHEADER *)buffer2);

 AVIStreamHeader *vid;
 vid=(AVIStreamHeader *)buffer;

  if (!fourCC::check( vid->fccType, (uint8_t *) "vids") &&getNbStream() > 1 )
  {
	 assert(_riff->
		   GetNextChunk((uint8_t *) "strl", (uint8_t *) "strh", &buffer,
				&len));
    	assert(_riff->
		   GetNextChunk((uint8_t *) "strl", (uint8_t *) "strf", &buffer2,
				&len2));

	Endian_AviStreamHeader((AVIStreamHeader *)buffer);
	Endian_BitMapInfo( (BITMAPINFOHEADER *)buffer2);
 	swapped=1;
  }

    attachVideo((AVIStreamHeader *) buffer, (BITMAPINFOHEADER *) buffer2,len2);
    free(buffer);
    free(buffer2);


	  if (getNbStream() > 2)
		{
         	if(GUI_Question((char *)"Take second audio stream?")) _audioStream=1;
        	}
//_________________________
// Read second stream (if exists)
//_________________________
    if (getNbStream() >= 2)
      {
      	if(swapped)
		{	//rewind
			_riff->rewindChunks();

		}
	  printf("\n Reading second stream information (2/%d)\n",getNbStream());

	  assert(_riff->
		 GetNextChunk((uint8_t *) "strl", (uint8_t *) "strh",
			      &buffer, &len));
	  assert(_riff->
		 GetNextChunk((uint8_t *) "strl", (uint8_t *) "strf",
			      &buffer2, &len2));

	if(_audioStream) //  write out second audio stream to first one
{
	  assert(_riff->
		 GetNextChunk((uint8_t *) "strl", (uint8_t *) "strh",
			      &buffer, &len));
	  assert(_riff->
		 GetNextChunk((uint8_t *) "strl", (uint8_t *) "strf",
			      &buffer2, &len2));
}

	Endian_AviStreamHeader((AVIStreamHeader *)buffer);
	Endian_WavHeader((WAVHeader *)buffer2);
	  attachAudio((AVIStreamHeader *) buffer, (void *) buffer2, len2);
    printf("\n Audio strf : %ld (wav %d)",len2,sizeof(WAVHeader));
	  free(buffer);
#warning FIXME
	  // free(buffer2);
      }
// if the flag has_index is NOT set force reindex
    if (_mainaviheader.dwFlags & AVIF_HASINDEX)
	printf("\n HAS INDEX");
    else
	printf("\n HAS NOT INDEX");

    if (_mainaviheader.dwFlags & AVIF_ISINTERLEAVED)
	printf("\n HAS INTERLEAVED");
    if (_mainaviheader.dwFlags & AVIF_TRUSTCKTYPE)
	printf("\n HAS TRUSTCKTYPE");

    _riff->dumpChunks();
    if (!(_mainaviheader.dwFlags & AVIF_HASINDEX))
	rebuild_index = 1;
//      
    if (rebuild_index)
	reIndex();
    else
      {

	  if (!index())
	      reIndex();	// if index fails reindex all the thing
      }
    // if audio...
	_avdm_stream = (AVDMAviAudioStream *) NULL;

    // finally check for missing FCC (mencoder)
    //if(_videostream.fccHandler==0)
    {
	_videostream.fccHandler = _video_bih.biCompression;
	printf("\n Warning no fcc, trying bih one...\n");
    }
    printf("\n Merging video index (%lu)\n",_nb_vchunk);
    res=mergeIndex(_nb_vchunk, _videochunks._next, &_videoindex);
    if(!res) return 0;
    printf(" Merging audio index (%lu)\n",_nb_achunk);
   res= mergeIndex(_nb_achunk, _audiochunks._next, &_audioindex);
    if(!res) return 0;
    if (_isaudiopresent)
      {
      	uint32_t extra;
	uint8_t *data;

		// we got some extra data in header
	  if(_audiostreaminfolen>(sizeof(WAVHeader)+2))
	  {
	  	data=(uint8_t *)_audiostreaminfo;
		extra=_audiostreaminfolen-sizeof(WAVHeader)-2;
		data+=sizeof(WAVHeader)+2;

	  }else{ extra=0;data=NULL;}
	  _avdm_stream = new AVDMAviAudioStream(_audioindex,
   						_nb_achunk,
						_riff,
						(WAVHeader *)_audiostreaminfo,						
						0,
						extra,data);

    }
    Dump();
	return 1;
}

//_________________________________________________
aviHeader::aviHeader(void)
{

    	_isaudiopresent = 0;
    	_isvideopresent = 0;
    	_maxaudio = _maxvideo = 0;
    	last_vchunk = NULL;
    	_curframe = 0;
	_audioStream=0;
	
}

//_____________________________
uint32_t aviHeader::getNbStream(void)
{
    return (uint32_t) _mainaviheader.dwStreams;

}

//
// Clumsy bit no 1
//_______________________________________________
void aviHeader::attachVideo(AVIStreamHeader * vid, BITMAPINFOHEADER * bih,uint32_t len)
{

    _isvideopresent = 1;
    if (!fourCC::check( vid->fccType, (uint8_t *) "vids"))
      {
	  printf( " First stream is *NOT* video \n");
	  fourCC::print(vid->fccType);
	  printf("\n");
	  assert(0);
      }
    memcpy(&_videostream, vid, sizeof(AVIStreamHeader));
    memcpy(&_video_bih, bih, sizeof(BITMAPINFOHEADER));
    _videostreaminfolen=len;
	
    if(len>sizeof(BITMAPINFOHEADER))
    {
    	uint8_t *in;
	
	in=(uint8_t *)bih;
	
	
    	_videoExtraLen=len-sizeof(BITMAPINFOHEADER);
	_videoExtraData=new uint8_t[_videoExtraLen];
	assert(_videoExtraData);
	memcpy(_videoExtraData,in+sizeof(BITMAPINFOHEADER),_videoExtraLen);
	printf("\n Added %ld bytes of extra data for video \n",_videoExtraLen);
        
    }
    
        
}

//_________________________________________________
//
// Clumsy bits no2
//_______________________________________________
void aviHeader::attachAudio(AVIStreamHeader * vid, void *bih, uint32_t len)
{

    _isaudiopresent = 1;
    if (!fourCC::check( vid->fccType, (uint8_t *) "auds"))
      {
	 printf( " Second stream is *NOT* audio \n");
	  fourCC::print(vid->fccType);
	  printf("\n");
	  assert(0);
      }
    memcpy(&_audiostream, vid, sizeof(AVIStreamHeader));
    _audiostreaminfo = bih;
    _audiostreaminfolen = len;

//              memcpy(&_video_bih,bih,sizeof(BITMAPINFOHEADER));

}

//_________________________________
//__________ RE-INDEXING __________
//_________________________________
// Ignore build-in index and climb
// the sub chunks to rebuild one
//
//  Warning -> In that case : no keyframe !!!!
//
uint8_t aviHeader::reIndex(void)
{
    SubChunk *vchunk, *achunk;
    uint32_t asize = 0, vsize = 0, fcc, len;
    int32_t togo;

    // FIXME : I should try to understand
    // how to read the index chunk

    printf(" Reindexing\n");

    vchunk = &_videochunks;
    achunk = &_audiochunks;
    _riff->rewindChunks();

    //assert(_riff->GoToPrimary((uint8_t *)"movi",&fcc));
    if (!_riff->
	GoToSecondary((uint8_t *) "movi", (uint8_t *) "dead",
		      (uint32_t *) & togo))
      {
	 printf( "***************************\n");
	printf( " Sorry i can't handle that for now\n");
	  printf( " and prefer to die rather than continue\n");
	
	  assert(0);

      }
    _nb_achunk = _nb_vchunk = 0;

    do
      {
	  //printf("\nIndexing running, audio :%lu video : %lu found",
	  //              _nb_achunk,_nb_vchunk);


	  fcc = _riff->readu32();
	  len = _riff->readu32();
	  togo -= 8;
#ifdef __DEBUG
	  printf("\n\t  pos : %lu (%lx) len : %lu (%lx) togo:%lu  FCC: ",
		 _riff->getpos(), _riff->getpos(), len, len, togo);
	  fourCC::print(fcc);

#endif
	  if (fourCC::check(fcc, (uint8_t *) "01wb"))
	    {

		// audio
		achunk = achunk->append(_riff->getpos(), len, 0);
		_riff->forward(len);
		togo -= len;
		_nb_achunk++;
		asize += len;
		if (len > _maxaudio)
		    _maxaudio = len;
	  } else
	    {
		if (
		(fourCC::check(fcc, (uint8_t *) "00dc"))
		    || (fourCC::check(fcc, (uint8_t *) "00db"))
		    || (fourCC::check(fcc, (uint8_t *) "01dc"))
		    || (fourCC::check(fcc, (uint8_t *) "01db"))

		    )
		  {		// nope, it is a video one
		      if (fourCC::check(fcc, (uint8_t *) "00db") ||
			fourCC::check(fcc, (uint8_t *) "01db")
		      )
			  vchunk =
			      vchunk->append(_riff->getpos(), len,
					     AVI_KEY_FRAME);
		      else
			  vchunk = vchunk->append(_riff->getpos(), len, 0);

		      vsize += len;
		      _riff->forward(len);
		      _nb_vchunk++;
		      togo -= len;
		      if (len > _maxvideo)
			  _maxvideo = len;
		} else
		  {
		      // stop when a non-a/v chunk is found
		      // we got them all ?

		      goto endofindex;
		  }
	    }


      }
    while (togo > 0);
  endofindex:
    printf("\nIndexing done, audio  :%lu video  : %lu found",
	   _nb_achunk, _nb_vchunk);
    printf("\n               ausize :%lu vdsize : %lu found\n",
	   asize, vsize);
    return 1;

}

//_________________________________
//__________ index __________
//
//  Return 1 if indexing succeeded, 0 otherwise
//_________________________________
// use build-in index

uint8_t aviHeader::index(void)
{
    uint32_t idxlen, len, i = 0;

    // we allow 3 audio tracks and 1 video tracks
    // should be enough


    SubChunk *cvchunk=&_videochunks;

    SubChunk achunk[3];
    SubChunk *cachunk[3]={NULL,NULL,NULL};
    uint32_t  nb_achunk[3]={0,0,0};

    uint32_t fcc,cctype, flags, abs, sync, delta = 0, c = 0;

   uint32_t trackId;

    printf("Indexing\n");

	_audiochunks._next=NULL;
	_nb_achunk=0;
	_videochunks._next=NULL;
	_nb_vchunk=0;

    if (!(_mainaviheader.dwFlags & AVIF_HASINDEX))
      {
        	printf("\n No index available ...\n");
	  return 0;

      }

    _riff->rewindChunks();

    assert(_riff->GoToPrimary((uint8_t *) "movi", &sync));
    sync = _riff->getpos();

    if (!	(_riff->	 GoToSecondary((uint8_t *) "movi", (uint8_t *) "idx1", &idxlen)))
      {				// there's a problem with the index
	  			_riff->rewindChunks();
	  			return 0;
      }
    // calibrate offset
    while (i < idxlen)
      {
	  fcc = _riff->readu32();
	  flags = _riff->readu32();
	  abs = _riff->readu32();
	  len = _riff->readu32();

	//fourCC::print(fcc);printf("\n");

	  if (i == 0)
	    {
		delta = 8 + sync - abs;	// ugly... but it works even
		// if avifile is not built correctly
		printf("\n Delta : %lu", delta);
		printf("\n sync : %lu", sync);
		printf("\n abs : %lu", abs);
	    }

	  trackId=((fcc>>8) & 0xFF) -'0';;
	  cctype=fcc >>16;


	switch(cctype)
	{
		case 'bw' : // audio track  wb
				if(nb_achunk[trackId]==0)
				{
					cachunk[trackId]=&achunk[trackId];
					printf("\n**We have a candidate for track %lu \n",trackId);
				}
					cachunk[trackId]->append(abs + delta, len, flags);
   					cachunk[trackId] = cachunk[trackId]->_next;
		      			nb_achunk[trackId]++;
					if (len > _maxaudio)
			  		_maxaudio = len;
				break;
		case 'cd':
		case 'bd': // DC / DB
   	      		c++;
		       cvchunk->append(abs + delta, len, flags);
		       cvchunk = cvchunk->_next;
		      _nb_vchunk++;
		      	if (len > _maxvideo)
			  _maxvideo = len;
		  	break;
	default:
		printf("\n Warning: Unknown FCC in index, skipping ->");
		fourCC::print(fcc);
		break;
	    }

	  i += 16;
      }
    printf("\n video chunk :%08lu audio chunk :%08lu", _nb_vchunk,
	   _nb_achunk);
    // search 1st audio stream
	SubChunk *first=NULL,*second=NULL;

	uint32_t f,s;

	for(f=0;f<3;f++)
	{
		if(cachunk[f])
		{
			first=achunk[f]._next;
			break;
		}
	}
	if(f<3)
	{
		for(s=f+1;s<3;s++)
		{
			if(cachunk[s])
			{
				second=achunk[s]._next;
				break;
			}
		}
	}
	printf(" First : %d , second : %d\n",f,s);
	// if _audio _audioStream=1 -> take second
	if(_audioStream)
	{
		assert(second);
		printf("Taking second audio stream \n");

		_audiochunks._next=second;
		_nb_achunk=nb_achunk[s];
		// purge first here !
		deleteChain(first);
	}
	else
	{
		if(first)
		{
			printf("Taking first audio stream \n");
			_audiochunks._next=first;
			_nb_achunk=nb_achunk[f];
			// purge second here !
			if(second)
			{
				deleteChain(second);
			}
		}
		else // no audio at all
		{
			_audiochunks._next=NULL;
			_nb_achunk=0;

		}

	}
	printf("%lu chunks  \n",_nb_achunk);

	return 1;

}


//_______________________________________
// Get simplified information for gui
//
//_____________________________________
WAVHeader *aviHeader::getAudioInfo(void)
{
    if (!_isaudiopresent)
      {
	  printf("\n ******************************no audio...\n");
	  return NULL;
      }
    // should put extra check here....
    printf("\n *** audio found (1) ***\n");
    return (WAVHeader *) & _audiostreaminfo;
//      memcpy(info,_audiostreaminfo,sizeof(WAVHeader));


}

uint8_t aviHeader::getAudioStream(AVDMGenericAudioStream ** audio)
{

    if (!_isaudiopresent)
      {
	  printf("\n ******************************no audio...\n");
	  *audio = (AVDMGenericAudioStream *) NULL;
	  return 0;
      }
    // should put extra check here....
    printf("\n *** audio found (2) ***\n");
    (*audio) = (AVDMGenericAudioStream *) _avdm_stream;
    return 1;
}

void aviHeader::deleteChain(SubChunk *first)
{


    SubChunk *old;
     while(first)
      {
	  old = first;
	  first = first->_next;
	  delete old;
      }
}



//EOF
