/***************************************************************************
                          ADM_mpegindexer.cpp  -  description
                             -------------------
		Mix between mpeg2dec code, vdub one and mine.         
		
		The audio stream is a bit complicated
		if C0 C1 ... mpeg audio directly
		if 0... 7 -> AC3 track no i, the conversion to sub stream ID / PRIVATE STREAM 1 will be done
						internally by the demuxer		                                        
                             
    begin                : Tue Oct 15 2002
    copyright            : (C) 2002 by mean
    email                : fixounet@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#ifdef __FreeBSD__
          #include <sys/types.h>
#endif
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include "config.h"
#include <math.h>

#include "ADM_library/default.h"
#include "avifmt.h"
#include "avifmt2.h"

#include "ADM_editor/ADM_Video.h"
#include "ADM_audio/aviaudio.hxx"

#include "ADM_library/fourcc.h"
#include "ADM_mpegindexer/ADM_mpegparser.h"
#include "ADM_mpeg2dec/ADM_mpegpacket.h"
#include "ADM_mpeg2dec/ADM_mpegpacket_PS.h"

#include "ADM_toolkit/toolkit.hxx"
#include "ADM_toolkit/filesel.h"
#include "ADM_dialog/DIA_working.h"

#include "ADM_toolkit/ADM_debugID.h"
#define MODULE_NAME MODULE_MPEG
#include "ADM_toolkit/ADM_debug.h"


void mpegToIndex2(char *name);
void mpegToIndex(char *name)  ;
//static void mpegPictureScan2(char *name);

uint8_t indexMpeg(char *mpeg,char *file);

static const uint32_t FPS[16]={
		0,			// 0
		23976,		// 1 (23.976 fps) - FILM
		24000,		// 2 (24.000 fps)
		25000,		// 3 (25.000 fps) - PAL
		29970,		// 4 (29.970 fps) - NTSC
		30000,		// 5 (30.000 fps)
		50000,		// 6 (50.000 fps) - PAL noninterlaced
		59940,		// 7 (59.940 fps) - NTSC noninterlaced
		60000,		// 8 (60.000 fps)
		0,			// 9
		0,			// 10
		0,			// 11
		0,			// 12
		0,			// 13
		0,			// 14
		0			// 15
	};

static uint8_t tryAudioTrack(char *name,uint8_t id);
uint8_t  MpegaudoDetectAudio(char *mpeg, uint8_t *audioTrack);

//static char *outname;
//static char *inname;
mParser *parser;
uint32_t nb_pic=0,skip=0,nb_aud=0;;
uint32_t sz_pic=0,sz_aud=0;

uint64_t pos_start=0;
FILE *_out;
uint8_t bfe[720*576*2];

uint8_t  MpegaudoDetectAudio(char *name, uint8_t *audioTrack)
{

    	uint8_t streamid,audiostreamid=0x00;
	mParser *parser;


	parser=new mParser();

	printf("audio Id: %x\n",audiostreamid);
	if(!parser->open(name))
		{
				GUI_Alert("Error reading mpeg !");
				delete parser;
				return 0;
		}
		uint8_t token;

	parser->sync(&token);

	// first identify stream type (ES/PS)
	delete parser;

	switch(token)
		{
				case 0xb3: // video es
									// no audio in video es
									printf("video es, no audio\n");
									return 0;
									break;
				case 0xba : // program stream ?	(vob/mpeg/VCD)
									break;
				default:
								printf("\n unrecognized stream\n");
								return 0;

		}

	printf("Program stream");
	// check for ac3 ,  i.e track 0

		if(tryAudioTrack(name,0x0))
		{
			printf("AC3\n");
			*audioTrack=0;return 1;
		}
		if(tryAudioTrack(name,0xc0))
		{
			printf("Mpeg\n");
			*audioTrack=0xc0;return 1;
		}
		if(tryAudioTrack(name,0xA0))
		{
			printf("LPCM\n");
			*audioTrack=0xA0;return 1;
		}
		printf("could not find audio track\n");
		return 0;
}
uint8_t tryAudioTrack(char *name,uint8_t id)
{
	uint32_t count=0,audio=0;
	uint8_t streamid;
	ADM_mpegDemuxer *demuxer;

	demuxer=new ADM_mpegDemuxerProgramStream(0xe0,id);
	demuxer->open(name);

	while(demuxer->sync(&streamid) && count<1000) count++;

	audio=demuxer->getOtherSize();
	delete demuxer;

	if(audio > 3000)
		{
				return 1;
		}
	return 0;
}

uint8_t indexMpeg(char *mpeg,char *file,uint8_t audioid)
{
	static const char Type[5]={'X','I','P','B','D'};
	FILE *out=NULL;
	uint32_t val;
	uint32_t hnt;;
//	uint32_t in;
	uint64_t pos;
	uint64_t abspos;

	uint32_t w,h,fps ;
	uint32_t nb_iframe=0;
//	uint32_t nb_pframe=0;
//	uint32_t nb_bframe=0;
//	uint32_t first=0;
	uint32_t total_frame=0;
	uint32_t seq_found=0;
	uint64_t lastI=0;
	uint32_t update=0;
    uint8_t streamid,audiostreamid=0x00;
	mParser *parser;
	ADM_mpegDemuxer *demuxer;

	audiostreamid=audioid;
	parser=new mParser();

	printf("audio Id: %x\n",audiostreamid);
	if(!parser->open(mpeg))
		{
				GUI_Alert("Error reading mpeg !");
				delete parser;
				return 0;
		}
		uint8_t token;

	parser->sync(&token);

	// first identify stream type (ES/PS)
	delete parser;

	switch(token)
		{
				case 0xb3: // video es
									printf("Video Elementary stream");
									demuxer=new ADM_mpegDemuxerElementaryStream;
									break;
				case 0xba : // program stream ?	(vob/mpeg/VCD)
									printf("Program stream");

									demuxer=new ADM_mpegDemuxerProgramStream(0xe0,audiostreamid);
									break;
				default:
								printf("\n unrecognized stream\n");
								return 0;
												
		}
	// ok now we have it.
	demuxer->open(mpeg);
	out=fopen(file,"wt");
	if(!out)
	{
			printf("\n Error : cannot open index !");
			delete demuxer;
			return 0;
	}
	fprintf(out,"IDXM ");
	if(token==0xb3) fprintf(out,"E XX\n");
	else   fprintf(out,"P XX\n");
	fprintf(out,"000000000000\n");
	fprintf(out,"1\n");
	fprintf(out,"%s\n",mpeg);

	DIA_working *work;

	work=new DIA_working("Indexing mpeg");

	while(1)
	{
				val=0;
				hnt=0;

				// sync on the video stream is done by demuxer, now get on the infos
				//
				if(!demuxer->sync(&streamid)) break;

				demuxer->getpos(&pos);
				abspos=demuxer->getAbsPos();
				update++;
			//	printf("\n sync found : %d",streamid);
				if(update>100)
					{
						if(work->update(demuxer->getAbsPos()>>16,demuxer->getSize()>>16))
						{
							// abort;
							goto stop_found;

						}
						update=0;
					}
				switch(streamid)
					{
					/* Useless apparently
						  	case 0xB9: // sequence end
                                        		printf("\n End seq\n");
                                        		goto stop_found;
                                        		break;
					*/
						  case 0xB3: // sequence start
											  if(seq_found)
											  	{
													 	demuxer->forward(8);
													  break;
													}
											  seq_found=1;
								             val=demuxer->read32i();
							   					w=val>>20;
							        			w=((w+15)&~15);
								        		h= (((val>>8) & 0xfff)+15)& ~15;
								      			fps= FPS[val & 0xf];
						    			        demuxer->forward(4);
						         			   printf("\n %ld x %ld at %ld fps\n",w,h,fps);
					           			 	fprintf(out,"%ld %ld %ld\n",w,h,fps);

						            break;
							case 0xb8: // GOP
											//	printf("GOP\n");
												demuxer->forward(4);
												break;
							case 0x00 : // picture
							                //  printf("PIC\n");
												uint8_t ftype;

												total_frame++;
												val=demuxer->read16i();
												ftype=7 & (val>>3);
												switch(ftype)
													{
														case 2:
														case 3:
														case 4:
																	if(total_frame==1)
																		{
																				total_frame=0;
																				continue;
																		}
														case 1:// printf(" Pic : I\n");
																		//	printf("\n%c",Type[ftype]);
																			if(total_frame!=1)
																				{
																	          fprintf(out," %llu\n",pos-lastI-4);

																				}


														          fprintf(out,"%c %010llX",Type[ftype],pos-4);
														          if(ftype==1) // I frame
														          {
																			  fprintf(out," %010llX %10lx",abspos-4,demuxer->getOtherSize()); 	
																	}
														          lastI=pos-4;
														          nb_iframe++;
																			break;

													}

												break;
							default:
												break;
					}
		}
stop_found:
//	 	fprintf(out," %d\n\n",demuxer->getSize()-lastI-4);
	 	fprintf(out," 0 \n\n");
       printf("\n end of stream...\n");
	  // update # of frames
	    fseek(out,0,SEEK_SET);
	  	fprintf(out,"IDXM ");
		if(token==0xb3) fprintf(out,"E %02x\n",audiostreamid);
			else   fprintf(out,"P %02x\n",audiostreamid);
		fprintf(out,"%012lX\n",nb_iframe);

		fclose(out);
		delete work;
	  printf("\n Total : %lu frames\n",total_frame);
	  delete demuxer;
	  return 1;
}

